import logging
import subprocess
from pathlib import Path


def install_script(path: Path):
    if (path/'pyproject.toml').exists():

        logging.debug(f'pyproject.toml found, running poetry install')
        p = subprocess.Popen(['poetry', 'install'], cwd=path.absolute())
        p.wait()
    else:
        logging.debug(f'No pyproject.toml found, skipping {path}')


import subprocess
from pathlib import Path


def install_script(path: Path):
    print(path)
    if (path/'package.json').exists():
        p = subprocess.Popen(['npm', 'install'], cwd=path.absolute())
        p.wait()



import importlib

import click
import gitlab


@gitlab.command()
@click.option('--group')
@click.option('--name')
@click.option('--script', type=click.Path(exists=True))
def run_script(client: GitlabClient, group, name, script):
    spec = importlib.util.spec_from_file_location("script", script)
    script = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(script)
    client.do_for_every_project(group, script.run)




__version__ = '0.1.0'

import importlib
import logging
import pkgutil

from devcli.context import DevCliContext
from devcli.devcli import cli

logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)


def load_plugins():
    return {
        name: importlib.import_module(name)
        for finder, name, ispkg
        in pkgutil.iter_modules()
        if name.startswith('devcli')
    }


plugins = load_plugins()


def run():

    cli(obj=DevCliContext.from_config_file())


from .gitlab_group import gitlab
from .gitlab_client import GitlabClient
from .sync import sync
from .add_file import add_file
from .trigger_pipelines import trigger_pipelines
from .edit_file import edit_file





import logging

import requests
from gitlab import Gitlab

from devcli import DevCliContext


class GitlabClient:

    def __init__(self, ctx: DevCliContext):
        self.token = ctx.config.get('gitlab.token')
        self.host = ctx.config.get('gitlab.host', 'https://gitlab.com')
        self.ctx = ctx
        logging.info(f'gitlab client configured with  {self.token}  and {self.host}')

        self.rest_client = Gitlab(url=self.host, private_token=self.token)
        self.graphql_headers = {"Authorization": f"Bearer {self.token}"}

    def graphql(self, query):
        request = requests.post(f'{self.host}/api/graphql', json={'query': query}, headers=self.graphql_headers)
        if request.status_code == 200:
            return request.json()
        else:
            raise Exception()

    def get_projects(self, group_path):
        query = f'''
          query {{
          group(fullPath: "{group_path}") {{
            id
            name
            projects(includeSubgroups: true) {{
              nodes {{
                id
                name
                fullPath
                sshUrlToRepo
              }}
            }}
          }}
        }}'''

        response = self.graphql(query)
        logging.info(f'response for group: {group_path}: {response}')
        return response['data']['group']['projects']['nodes']

    def do_for_every_project(self, group, func, fail_on_failure=False):

        for project in self.get_projects(group):
            gl_project = self.rest_client.projects.get(project['fullPath'], lazy=True)
            logging.info(f'processing project: {gl_project.id}')
            try:
                func(gl_project)
            except Exception as e:
                logging.error(f'error during project {gl_project.id}', e)



import click
from gitlab import GitlabGetError

from . import GitlabClient
from .gitlab_group import gitlab


@gitlab.command()
@click.option('--group')
@click.option('--name')
@click.argument('file', type=click.File('r'))
@click.pass_obj
def add_file(client: GitlabClient, group, name, file):
    contents = file.read()
    client.do_for_every_project(group, add_file_function(name, contents))


def add_file_function(file_name, file_contents):
    def add_file_to_project(project):

        try:
            f = project.files.create({'file_path': file_name,
                                      'branch': 'master',
                                      'content': file_contents,
                                      # 'author_email': 'job.denoo@dearhealth.com',
                                      # 'author_name': 'Job de Noo',
                                      'commit_message': 'added pytest'})

        except GitlabGetError as e:
            print(e)
            if e.response_code == 404:
                print('contunueing')
                pass
            else:
                raise e
    return add_file_to_project



import asyncio
import logging
from pathlib import Path

import click

from devcli_gitlab import gitlab, GitlabClient
from devcli_gitlab.git_utils import clone_or_pull_project


async def sync_tree(client: GitlabClient, group_name, install_scripts):
    projects = client.get_projects(group_name)
    await asyncio.gather(
        *[sync_repo(group_name, project, install_scripts) for project
          in
          projects])


async def sync_repo( group_name, project, install_scripts):

    relative_path = project['fullPath'].replace(f'{group_name}/', '')
    path = Path(relative_path)
    path.mkdir(parents=True, exist_ok=True)
    await clone_or_pull_project(path, project['sshUrlToRepo'])
    print('running install script')
    [install_script(path) for install_script in install_scripts]


# make pluggable
@gitlab.command()
@click.pass_obj
@click.option('--group')
@click.option('--install/--noinstall', default=False)
def sync(client: GitlabClient, group, install):
    print(install)
    install_scripts = client.ctx.get_plugin_scripts('install_script') if install else []
    logging.info(f'loaded install scripts: {install_scripts}')
    asyncio.run(sync_tree(client, group, install_scripts))



import importlib
import os
import pkgutil

import yaml
from cachetools.func import lru_cache


class DevCliContext:

    def __init__(self, config) -> None:
        self.config = config

        self.load_plugins()

    def load_plugins(self):
        self.plugins = {
            name: importlib.import_module(name)
            for finder, name, ispkg
            in pkgutil.iter_modules()
            if name.startswith('devcli')
        }

    @lru_cache
    def get_plugin_scripts(self, script_name):
        print(self.plugins)
        return [getattr(plugin, script_name) for plugin in self.plugins.values() if hasattr(plugin, script_name)]

    def from_config_file():
        with open(os.path.join(os.getenv("HOME"), '.devcli_config')) as file:
            config = yaml.load(file)
            return DevCliContext(config)



from typing import List

import click

from . import GitlabClient
from .gitlab_group import gitlab


@gitlab.command(short_help='triggers the pipeline for all projects in group')
@click.pass_obj
@click.option('--group')
@click.option('--variables', '-m', multiple=True, type=click.Tuple([str, str]),default=[])
@click.option('--include-token-under', default=None)
def trigger_pipelines(client: GitlabClient, group, variables:List, include_token_under):
    variables = [{'key':variable[0] , 'value':variable[1]} for variable in variables]
    if include_token_under:
        variables.append({'key': include_token_under, 'value': client.token})

    client.do_for_every_project(group, lambda project: project.pipelines.create({'ref': 'master', 'variables': variables}))




import logging
import pathlib
import sys

import git


def is_git_repo(path):
    try:
        _ = git.Repo(path).git_dir
        return True
    except git.InvalidGitRepositoryError:
        return False


async def clone_or_pull_project(path: pathlib.Path, url):
    if is_git_repo(path):
        pull_repo(path)
    else:
        clone_project(path, url)


def clone_project(path, url):
    logging.info(f"cloning project: { path.absolute()}")
    try:
        git.Repo.clone_from(url, path)
    except KeyboardInterrupt:
        logging.error("User interrupted")
        sys.exit(0)
    except Exception as e:
        logging.error("Error cloning project %s", path)
        logging.error(e)


def pull_repo(path):
    logging.info("updating existing project %s", path)
    try:
        repo = git.Repo(path)
        repo.remotes.origin.pull()
        logging.info("done updating project  %s", path)

    except KeyboardInterrupt:
        logging.error("User interrupted")
        sys.exit(0)
    except Exception as e:
        logging.error("Error pulling project %s", path)
        logging.error(e)





import base64
import importlib

import click
from gitlab import GitlabGetError

from devcli_gitlab import GitlabClient
from devcli_gitlab.gitlab_group import gitlab


@gitlab.command()
@click.option('--group')
@click.option('--name')
@click.option('--script', type=click.Path(exists=True))
@click.option('--message')
@click.option('--dry_run/--save', default=False)
@click.pass_obj
def edit_file(client: GitlabClient, group, name, script, message, dry_run):

    client.do_for_every_project(group, edit_file_function(name, script, message, dry_run))


def edit_file_function(file_name, script, message, dry_run):

    spec = importlib.util.spec_from_file_location("edit_script", script)
    edit_script = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(edit_script)

    def edit_file(project):
        try:
            f = project.files.get(file_path=file_name, ref='master')
            f.content = f.decode()
            # f.content = base64.b64decode(f.content).decode('utf-8')

            def save_cb():
                print('in save_cb')
                if dry_run:
                    print('-------------altered file content -----------')
                    print(f.content)
                else:
                    f.save(branch='master', commit_message=message)

            edit_script.edit_file(project, f, save_cb)


        except GitlabGetError as e:
            print(e)
            if e.response_code == 404:
                print('file not in repo ,continuing')
                pass
            else:
                raise e

    return edit_file




import logging

import click


@click.group()
@click.pass_context
@click.option('--verbose/--verbose-of', default=False)
def cli(ctx, verbose):
    print(ctx.obj)
    if verbose:
        logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.DEBUG)





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['devcli', 'devcli_gitlab', 'devcli_npm_install', 'devcli_poetry_install']

package_data = \
{'': ['*']}

install_requires = \
['cachetools>=4.1.1,<5.0.0',
 'click>=7.1.2,<8.0.0',
 'python-gitlab>=2.5.0,<3.0.0',
 'python-graphql-client>=0.4.0,<0.5.0',
 'ruamel.yaml>=0.16.12,<0.17.0']

entry_points = \
{'console_scripts': ['devcli = devcli:run']}

setup_kwargs = {
    'name': 'devcli',
    'version': '0.1.5',
    'description': '',
    'long_description': None,
    'author': 'Job de Noo',
    'author_email': 'jgdenoo@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/devnoo/dev-cli/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
